@@ -11,6 +11,7 @@ import shlex
 import subprocess
 import sys
 import threading
+import time
 import typing
 
 import joblib
@@ -65,7 +66,15 @@ CONFIG: dict[str, typing.Any] = {
     "extend-exclude": [],
 }
 
-ALL_COMMANDS = ["pyupgrade", "isort", "black", "pflake8", "mypy", "pylint", "pytest"]
+ALL_COMMANDS = {
+    "pyupgrade": {"type": "formatter"},
+    "isort": {"type": "formatter"},
+    "black": {"type": "formatter"},
+    "pflake8": {"type": "linter"},
+    "mypy": {"type": "linter"},
+    "pylint": {"type": "linter"},
+    "pytest": {"type": "tester"},
+}
 
 NCOLS = 128
 
@@ -122,6 +131,11 @@ def run(args: typing.Sequence[str] = None) -> int:
             )
         )
         return 0
+    # check
+    commands = args.commands.split(",")
+    for command in commands:
+        if command not in CONFIG:
+            parser.error(f"command not found: {command}")
 
     # pyproject.toml
     pyproject_path = pathlib.Path("pyproject.toml").absolute()
@@ -146,30 +160,37 @@ def run(args: typing.Sequence[str] = None) -> int:
             CONFIG[key] = value
 
     # run
-    jobs: typing.Any = []
-    for command in args.commands.split(","):
-        if command not in CONFIG:
-            parser.error(f"command not found: {command}")
-        if CONFIG[command]:
-            jobs.append(joblib.delayed(run_command)(command, args))
-    with joblib.Parallel(n_jobs=len(jobs), backend="threading") as parallel:
-        results = parallel(jobs)
+    results = _run_commands(commands, args)
 
     # summary
     logger.info(f"{'-' * 10} summary {'-' * (72 - 10 - 9)}")
     for result in results:
-        if result.returncode == 0:
-            status = "succeeded"
-        elif result.command in ("pyupgrade", "isort", "black"):
-            status = "formatted"
-        else:
-            status = "failed"
-        logger.info(f"    {result.command:<16s} {status} ({result.files}files)")
+        logger.info(f"    {result.command:<16s} {result.get_status_text()}")
     logger.info("-" * 72)
 
     return 0 if all(result.returncode == 0 for result in results) else 1
 
 
+def _run_commands(commands, args):
+    """コマンドの実行。"""
+    results: list[CommandResult] = []
+
+    # run formatters (serial)
+    for command in commands:
+        if CONFIG[command] and ALL_COMMANDS[command]["type"] == "formatter":
+            results.append(run_command(command, args))
+
+    # run linters/testers (parallel)
+    jobs: list[typing.Any] = []
+    for command in commands:
+        if CONFIG[command] and ALL_COMMANDS[command]["type"] != "formatter":
+            jobs.append(joblib.delayed(run_command)(command, args))
+    with joblib.Parallel(n_jobs=len(jobs), backend="threading") as parallel:
+        results = parallel(jobs)
+
+    return results
+
+
 @dataclasses.dataclass
 class CommandResult:
     """コマンドの実行結果。"""
@@ -177,6 +198,17 @@ class CommandResult:
     command: str
     returncode: int
     files: int
+    elapsed: float
+
+    def get_status_text(self):
+        """文字列化。"""
+        if self.returncode == 0:
+            status = "succeeded"
+        elif self.command in ("pyupgrade", "isort", "black"):
+            status = "formatted"
+        else:
+            status = "failed"
+        return f"{status} ({self.files}files in {self.elapsed:.1f}s)"
 
 
 def run_command(command: str, args: argparse.Namespace) -> CommandResult:
@@ -191,6 +223,7 @@ def run_command(command: str, args: argparse.Namespace) -> CommandResult:
     check_args = ["--check"] if command in ("black", "isort") else []
 
     # 実行
+    start_time = time.perf_counter()
     env = os.environ.copy()
     env["PYTHONIOENCODING"] = "utf-8"
     proc = subprocess.run(
@@ -217,6 +250,7 @@ def run_command(command: str, args: argparse.Namespace) -> CommandResult:
             errors="backslashreplace",
         )
     output = proc.stdout.strip()  # 再実行時の出力を採用
+    elapsed = time.perf_counter() - start_time
 
     # 結果表示
     mark = "*" if returncode == 0 else "@"
@@ -231,7 +265,9 @@ def run_command(command: str, args: argparse.Namespace) -> CommandResult:
         logger.info(f"{mark} returncode: {returncode}")
         logger.info(mark * NCOLS)
 
-    return CommandResult(command=command, returncode=returncode, files=len(targets))
+    return CommandResult(
+        command=command, returncode=returncode, files=len(targets), elapsed=elapsed
+    )
 
 
 def _expand_globs(targets: list[pathlib.Path], globs: list[str]) -> list[pathlib.Path]:
